unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComServ;

const
  WM_AFTERSHOW = WM_USER + 1234;
  WM_LOKSIMSIMISRUNNING = WM_USER + 12345;
  WM_LOKSIMSTARTWAIT = WM_USER + 12346;
  WM_LOKSIMSTOPWAIT = WM_USER + 12347;
  WM_LOKSIMSENDKEY = WM_USER + 12348;

  cGetSimData = 1;
  cSetSimData = 2;

type
  TFormCommunication = class(TForm)
    procedure Button1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private-Deklarationen }
    LoksimWnd: Hwnd;
    HasMessage: boolean;
    LastMessageString: WideString;
    function GetLoksimHwnd(): HWnd;
    procedure SendCopyDataEvent( EventId: DWORD; Value: WideString);
    procedure WMCopyData(var Msg: TWMCopyData); message WM_COPYDATA;
    procedure WMAfterShow( var Msg: TMessage); message WM_AFTERSHOW;
  public
    { Public-Deklarationen }
    function LoksimIsRunning: boolean;
    procedure StartWaitForCommands(TimeOut: Integer);
    procedure StopWaitForCommands;
    function SimulationIsRunning: boolean;
    function GetSimData(const ParamStr: WideString): WideString;
    procedure SendKey(const KeyIndex: integer);
  end;

var
  FormCommunication: TFormCommunication;

implementation

{$R *.dfm}



function TFormCommunication.LoksimIsRunning: boolean;
begin
  if 0 <> GetLoksimHwnd() then
  begin
    Result := True;
  end
  else
  begin
    Result := False;
  end;
end;


procedure TFormCommunication.Button1Click(Sender: TObject);
var
  S: wideString;
begin
  //if SimulationIsRunning() then
  begin
    S := GetSimData( 'LOK.SPEED');
    ShowMessage( S);
  end;
end;

function TFormCommunication.GetLoksimHwnd: HWnd;
begin
  if not IsWindow( LoksimWnd) then
  begin
    LoksimWnd := FindWindow( nil, 'LoksimComWindow A123456TZAR5');
  end;
  Result := LoksimWnd;
end;


function TFormCommunication.SimulationIsRunning: boolean;
begin
  Result := False;
  if LoksimIsRunning() then
  begin
    if 1 = SendMessage( LoksimWnd, WM_LOKSIMSIMISRUNNING, 0, 0) then
    begin
      Result := True;
    end;
  end;
end;

procedure TFormCommunication.StartWaitForCommands(TimeOut: Integer);
begin
  if LoksimIsRunning() then
  begin
    SendMessage( LoksimWnd, WM_LOKSIMSTARTWAIT, TimeOut, 0);
  end;
end;

procedure TFormCommunication.StopWaitForCommands;
begin
  if LoksimIsRunning() then
  begin
    SendMessage( LoksimWnd, WM_LOKSIMSTOPWAIT, 0, 0);
  end;
end;

function TFormCommunication.GetSimData(
  const ParamStr: WideString): WideString;
begin
  Result := '';
  HasMessage := False;
  SendCopyDataEvent( cGetSimData, ParamStr);
  if HasMessage then
  begin
    Result := LastMessageString;
  end;

end;


function DWStrAllocW( Size: Cardinal): PWideChar;
begin
  Size := SizeOf(WideChar) * Size + SizeOf(Cardinal);
  GetMem(Result, Size);
  FillChar(Result^, Size, 0);
  Cardinal(Pointer(Result)^) := Size;
  Inc(Result, SizeOf(Cardinal) div SizeOf(WideChar));
end;


procedure DWStrMoveW( Dest: pWideChar; Source: pWideChar; Size: cardinal);
Begin
  Move( Source^, Dest^, Size * sizeof(WideChar) );
End;

function DWStrLenW( S: pWideChar): integer;
Begin
  Result := 0;
  while ord( S[Result]) <> 0 do
  Begin
    Inc( Result);
  End;
End;


// Diese Funktion kann auch mit normalen Strings aufgerufen werden, wogegen
// StrNewW immer einen WideString verlangt, weil der cast PWideChar( S) ins Nirvana geht
function DWStrNewW(const S: WideString): pWideChar;
var
  Size: Cardinal;
  Str: pWideChar;
begin
  Str := pWideChar( S);
  if Str = nil then
    Result := nil
  else
  begin
    Size := DWStrLenW(Str) + 1;
    Result := DWStrAllocW(Size);
    DWStrMoveW( Result, Str, Size);
  end;
end;

procedure DWStrDisposeW(Str: PWideChar);
begin
  if Str <> nil then
  begin
    Dec(Str, SizeOf(Cardinal) div SizeOf(WideChar));
    FreeMem(Str, Cardinal(Pointer(Str)^));
  end;
end;


procedure TFormCommunication.SendCopyDataEvent(EventId: DWORD;
  Value: WideString);
var
	cpData: TCOPYDATASTRUCT;
  pString: pWideChar;
begin
  if LoksimIsRunning() then
  begin
    cpData.dwData := EventID;
    cpData.cbData := SizeOf(WideChar) * Length( Value) + SizeOf(Cardinal);
    pString := DWStrNewW( Value);
    cpData.lpData := pString;
	  SendMessage( LoksimWnd, WM_COPYDATA, Handle, LPARAM(@cpData));
    DWStrDisposeW( pString);
  end;
end;

procedure TFormCommunication.WMCopyData(var Msg: TWMCopyData);
var
  pString: pWideChar;
begin
  pString := Msg.CopyDataStruct.lpData;
  if Assigned( pString) then
  begin
    HasMessage := True;
    LastMessageString := pString;
    UniqueString( LastMessageString);
  end;
end;

procedure TFormCommunication.SendKey(const KeyIndex: integer);
begin
  if LoksimIsRunning() then
  begin
    SendMessage( LoksimWnd, WM_LOKSIMSENDKEY, KeyIndex, 0);
  end;
end;

procedure RegWriteString(const RootKey: HKEY; const Key, Name, Value: string);
var
  RegKey: HKEY;
begin
  if RegOpenKeyEx(RootKey, pChar(Key), 0, KEY_SET_VALUE, RegKey) = ERROR_SUCCESS then
  begin
    RegSetValueEx(RegKey, PChar(Name), 0, REG_SZ, PChar(Value), Length(Value)+1);
    RegCloseKey(RegKey);
  end;
end;


procedure TFormCommunication.FormCreate(Sender: TObject);
begin
  try
    // Trage als interaktiver User ein
    RegWriteString( HKEY_CLASSES_ROOT, 'AppID\{64CF4CD3-FA44-4606-B7FD-279BB59C51CA}', 'RunAs', 'Interactive User');
  Except
  end;

  if ComServer.StartMode in [smStandalone, smRegServer] then
  begin
    ShowMessage( 'LoksimControl wurde registriert');
    PostMessage( Handle, WM_AFTERSHOW, 1, 0);
  end
  else
  if ComServer.StartMode = smUnregServer then
  begin
    ShowMessage( 'LoksimControl wurde aus der Registrierung entfernt');
    PostMessage( Handle, WM_AFTERSHOW, 1, 0);
  end
  else
  begin
    PostMessage( Handle, WM_AFTERSHOW, 0, 0);
  end;
end;

procedure TFormCommunication.WMAfterShow(var Msg: TMessage);
begin
  if Msg.WParam = 0 then
  begin
    Visible := False;
  end
  else
  begin
    Close;
  end;
end;

end.
